import pandas as pd
import numpy as np

def bol_imputer(df,col,method='mode'): #How is the user converting their data into a df? 
    """
    Fill the empty values of a specific column with imputed values to facilitate data preprocessing.

    Parameters
    ----------
    df: pandas dataframe
        A pandas dataframe waiting for imputing
    col: string
        name of the column waiting for imputing
    method: string
        A name of the imputing strategy. They include ['mode'] or most frequent value.
    Returns
    ----------
    y: pandas dataframe
        A pandas dataframe after imputing

    Examples
    ----------
    >>> from simpute import bol_imputer
    >>> bol_imputer(row_df,'if_default_last_month','knn')
    """
    
    bool_col = df[col]

    #Replace any empty fields with nan
    bool_col.replace('', np.nan, inplace=True)

    #Check if the column does have null values
    if bool_col.isnull().sum() == 0:
        return print("The selected column does not have any missing values to impute.")

    #Check if the column is boolean
    if (bool_col.dropna().isin([True, False, 1, 0, 'True', 'False'])).all() == False:
        return print("The selected column is not Boolean, please select another column.")

    #Calculate frequencies of boolean values
    count_true = 0
    count_false = 0

    for bool in bool_col.dropna():
        if bool in [True, 'True', 1]:
            count_true += 1
        else:
            count_false += 1
    
    freq_true = count_true/len(bool_col.dropna())
    freq_false = count_false/len(bool_col.dropna())

    #Select high frequency value
    if freq_true == freq_false or freq_true > freq_false:
        impute_value = True 
    else:
        impute_value = False
    
    #Replace null values with 
    df[col].fillna(impute_value, inplace = True)

    return df




import pandas as pd
import numpy as np
from sklearn.feature_extraction import FeatureHasher
from sklearn.preprocessing import StandardScaler
from sklearn.preprocessing import OneHotEncoder
from sklearn.compose import make_column_transformer
from sklearn.model_selection import cross_val_score, cross_validate, train_test_split
from sklearn.pipeline import Pipeline, make_pipeline
from sklearn.preprocessing import OneHotEncoder, OrdinalEncoder, StandardScaler
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.neighbors import KNeighborsClassifier
from sklearn.preprocessing import LabelEncoder
from sklearn.impute import KNNImputer
class KNN_num_imputer:
    def __init__(self,df=None):
        self.df=df
        self.dtype_dic=None
    def remove_id(self,df,numeric=[]):
        for col in numeric:
            if set(df[col]==sorted(df[col]))=={True}:
                # print(df[col])
                numeric.remove(col)
        return numeric
    def if_text(self,element):
        count=0
        for i in element:
            if i.isdigit():
                count+=1
        return count/len(element)<0.5
    def dtype_groupby(self,df,categoric=[]):
        numeric=list(df.select_dtypes(include=[np.number]).columns.values)
        numeric=self.remove_id(df,numeric)
        remain=list(df.columns)
        # remain.remove(target)
        text=[]
        drop=[]
        for i in numeric+categoric:
            remain.remove(i)
        for j in remain:
            if isinstance(df[j][0],(str)) and self.if_text(df[j][0]):
                text.append(j)
            else:
                drop.append(j)
        return {'numeric':numeric,'text':text,'drop':drop,'categoric':categoric} 
    def fit_transform(self,categoric=[]):
        #dtype group by
        df=self.df
        dtype_dic=self.dtype_groupby(df,categoric)
        numeric_feature,categoric_feature, drop_feature,text_feature=dtype_dic['numeric'],dtype_dic['categoric'],dtype_dic['drop'],dtype_dic['text']
        
        ##preporcessor for numeric
        preprocessor = make_column_transformer(
        (StandardScaler(), numeric_feature),
        # (CountVectorizer(stop_words="english"), text_feature),
        (OneHotEncoder(),categoric_feature),
        ("drop", drop_feature))
        X_train_proced=preprocessor.fit_transform(df)
        if isinstance(X_train_proced,(np.ndarray)):
            df_proced=pd.DataFrame(X_train_proced,columns=preprocessor.get_feature_names_out())
        else:
            df_proced=pd.DataFrame(X_train_proced.todense(),columns=preprocessor.get_feature_names_out())
        imputer = KNNImputer(n_neighbors=3)
        filled_df=imputer.fit_transform(df_proced)
        filled_df=pd.DataFrame(filled_df,columns=preprocessor.get_feature_names_out())
        preprocessor.output_indices_
        result=df.copy()
        result[numeric_feature] =filled_df.iloc[:,preprocessor.output_indices_['standardscaler']]
        # result.isnull().sum()
        self.dtype_dic=dtype_dic
        return result
    def get_dtype_dic_out(self):
        return self.dtype_dic

class Sun_switch(object):
    def __init__(self, value):
        self.value = value
        self.fall = False
 
    def __iter__(self):
        """Return the match method once, then stop"""
        yield self.match
        raise StopIteration
    
    def match(self, *args):
        """Indicate whether or not to enter a case suite"""
        if self.fall or not args:
            return True
        elif self.value in args: # changed for v1.5, see below
            self.fall = True
            return True
        else:
            return False
def num_imputer(x,col,method=None):
    """
    Fill the empty values of a specific column with imputed values to facilitate data preprocessing.
    Parameters
    ----------
    x: pandas dataframe
        A pandas dataframe for imputing
    col: string
        name of the column to be imputed
    method: string
        A name of the imputing strategy. They include ['knn','mean','median','mode'], the default method is mean.
    Returns
    ----------
    y: pandas dataframe
        A pandas dataframe after imputing
    Examples
    ----------
    >>> from simpute import num_imputer
    >>> num_imputer(row_df,'price','knn')
    """
    #set default method
    if method==None or method=='mean':
        target_name=col
        target_col=x[target_name]
        emp_index=target_col[target_col.isnull()].index
        emp_df=x.iloc[emp_index,:]
        result=x.copy()
        result.iloc[emp_index,:]=x[target_name].mean()
        return result
    
    v=method
    for case in Sun_switch(v):
        if case('median'):
            target_name=col
            target_col=x[target_name]
            emp_index=target_col[target_col.isnull()].index
            emp_df=x.iloc[emp_index,:]
            result=x.copy()
            result.iloc[emp_index,:]=x[target_name].median()
            return result
            break
        if case('mode'):
            target_name=col
            target_col=x[target_name]
            emp_index=target_col[target_col.isnull()].index
            emp_df=x.iloc[emp_index,:]
            result=x.copy()
            result.iloc[emp_index,:]=x[target_name].mode()[0]
            return result
            break
        if case('knn'):
            KNN= KNN_num_imputer(x)
            return KNN.fit_transform()
            break
        if case(): 
            #default mean impute method
            print("method value wrong,the recent methd are ['knn','mean','median','mode'], the default method is mean.")
            # No need to break here, it'll stop anyway
            
def testData_reader(path):
    df1=pd.read_csv(path)
    return df1



import pandas as pd
import numpy as np

def cat_imputer(x,col):
    """
    Fill the empty values of a specific column with the most frequent category to facilitate data preprocessing.

    Parameters
    ----------
    x: pandas dataframe
        A pandas dataframe waiting for imputing
    col: string
        name of the column waiting for imputing
       Returns
    ----------
    y: pandas dataframe
        A pandas dataframe after imputing

    Examples
    ----------
    >>> from simpute import cat_imputer
    >>> cat_imputer(row_df,'age_group')
    """

    # check for bad input
    if not isinstance(x, pd.DataFrame):
        raise TypeError("Please pass in a Pandas DataFrame for `x`")
    if x.empty:
        raise ValueError("DataFrame cannot be empty")
    if col not in x.columns:
        raise Exception("The given column does not exist in the dataframe.")

    x[col].fillna(value=x[col].
                  value_counts().
                  index[0],
                  inplace =True)
    return x



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['simpute_py']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=1.5,<2.0', 'scikit-learn>=1.1,<2.0']

setup_kwargs = {
    'name': 'simpute-py',
    'version': '0.0.4',
    'description': 'A python package for simple data imputation',
    'long_description': '![](logo.png)\n\n### What does it do?\nHave you ever had a time when your missing data was holding you back? Well then this package is for you!\n\nOur python package for simple data imputation will allow you to quickly and seamlessly impute any missing data (be numeric, categorical, date/time or boolean values) using any large datasets.\n\nAll you have to do is follow these simple 4 steps:\n 1. Import the package and the data you wish you impute\n 2. Select the function and method for imputation (this will depend on the data type - read the usage section below for more details)\n 3. Hit run\n 4. Save your newly imputed dataset\n\nOur package will help simplify all your imputation needs so your data is ready when you need it!\n\n## Contributors & Maintainers\n- [Lisa Sequeira](https://github.com/LisaSeq)\n- [Renee Kwon](https://github.com/renee-kwon)\n- [Fujie Sun](https://github.com/Althrun-sun)\n- [Ken Wang](https://github.com/kenuiuc)\n\n## Installation\n\n```bash\n$ pip install simpute_py\n```\n\n## Usage\n\nWe have four main functions dealing with each data type:\n- `Num_imputer`: This function fills in the empty values of a numeric column with values derived from your selected imputation method. Your options for method include _knn_ (autogenerated values based on KNN), _mean_, _median_ and _mode_.\n- `Cat_imputer`: This function fills in the empty values of a categorical column with values derived based on most frequent (mode) category.\n- `Bol_imputer`: This function fills in the empty values of a boolean column with values derived using most frequent (mode) boolean value.\n- `Date_imputer`: This function fills in empty values of a date column with median point of the range of dates in that column.\n\n## Place in the Python Ecosystem\n\nCurrently, there are many other ways you can impute a dataset, using various functions build within Python, but this packages it neatly into one place and simplifies the process. We do have other packages you can use such as [AutoImpute](https://pypi.org/project/autoimpute/) and [MIDASpy](https://github.com/MIDASverse/MIDASpy). However our package aims to provide functionality not provided in either package and for more general audience use.\n\n## Contributing\n\nInterested in contributing? Check out the contributing guidelines. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`simpute_py` was created by Lisa Sequeira, Renee Kwon, Fujie Sun, and Ken Wang. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`simpute_py` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n\n',
    'author': 'Ken Wang',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
